# -*- coding: UTF-8 -*-
TEXT_EXTENSIONS = (
    # ASP Classic
    "asp",

    # ASP.NET
    "aspx",
    "axd",
    "asx",
    "asmx",
    "ashx",

    # CSS
    "css",

    # Coldfusion
    "cfm",

    # Erlang
    "yaws",

    # # Flash, treat as data
    # "swf",

    # HTML
    "html",
    "htm",
    "xhtml",
    "jhtml",
    "hta",

    # Java
    "jsp",
    "jspx",
    "wss",
    "do",
    "action",

    # JavaScript
    "js",

    # Perl
    "pl",

    # PHP
    "php",
    "php4",
    "php3",
    "phtml",

    # Python
    "py",

    # Ruby
    "rb",
    "rhtml",

    # SSI
    "shtml",

    # Other (C, perl etc.)
    # NOTE: these could also be data
    "cgi",
    "dll",

    # ASP.NET
    "axd",
    "asx",
    "asmx",
    "ashx",
    "aspx",

    # XML/Feed
    "xml",
    "rss",
    "atom",
    "svg",
)
